; Activity selection menu for Action 53
; Copyright 2012 Damian Yerrick
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.

.include "src/nes.h"
.include "src/ram.h"

.import linebuf
.import vwfStrWidth, vwfGlyphWidth, vwfPuts
.import clearLineImg, copyLineImg, invertTiles
.import stop_music, init_music, update_sound
.export cart_menu, get_titledir_a
.import ppu_clear_nt
.import unpb53_some, PB53_outbuf, interbank_fetch, interbank_fetch_buf
.importzp ciSrc, ciDst, ciBufStart, ciBufEnd

.exportzp ciSrc0, ciSrc1
.export draw_progress
ciSrc0 = tab_data_ptr
ciSrc1 = desc_data_ptr

SCREENSLIST = $FF04
TITLELIST = $FF06
PAGELIST = $FF08
NAMESLIST = $FF0A
DESCSLIST = $FF0C
DESCSBANK = $FF0E

MAX_TABS = 8
.segment "ZEROPAGE"
; During setup this points to the name of a tab.  But while the
; menu is running it points to the start of the tab->title table.
tab_data_ptr: .res 2

; Pointer to the data of the screenshot or description being
; accessed.
desc_data_ptr: .res 2

.segment "BSS"
tab_tilelens: .res MAX_TABS
tab_xoffsets: .res MAX_TABS
tab_title_offsets: .res MAX_TABS
screenshot_colors: .res 3
draw_step: .res 1
draw_progress: .res 1
num_pages: .res 1
cur_page: .res 1
cur_titleno: .res 1
showing_description: .res 1
screenshot_titleno: .res 1
num_lines_on_page: .res 1
prev_lines_on_page: .res 1

DRAWS_IDLE = 0
DRAWS_TABS = 2
  ; single frame: draw borders under tabs
  ; next step TITLELIST or DESCRIPTION
DRAWS_TITLELIST = 4
  ; draw_progress is line number (0-19)
  ; next step ARROWCURSOR
DRAWS_DESCRIPTION = 6
  ; draw_progress 0: title; 1: (C) year author; 2: number of players;
  ; 3: blank row; 4+: description
  ; next step ARROW
DRAWS_ARROW = 8
  ; single frame: draw arrow to left of title list
  ; next step SCREENSHOT
DRAWS_SCREENSHOT = 10
  ; draw_progress 0-6: tile row to decompress to $1000-$137F
  ; next step IDLE

.segment "CODE"
.proc cart_menu
  jsr setup_card_bg
  lda #0
  sta cur_page
  sta cur_titleno
  sta num_lines_on_page
  sta prev_lines_on_page
  sta showing_description
  lda #$FF
  sta screenshot_titleno

  jsr update_sound
  lda #DRAWS_TABS
  sta draw_step
  lda PAGELIST
  sta tab_data_ptr
  lda PAGELIST+1
  sta tab_data_ptr+1
  jsr update_sound

.if 0
  lda #1
  jsr init_music
.else
  jsr stop_music
.endif

forever:
  lda #0  ; if we were using sprite 0 hit, this would be #4
  sta oam_used
  lda screenshot_titleno
  cmp cur_titleno
  bne no_screenshot
  jsr setup_oam_for_screenshot
no_screenshot:
  ldx oam_used
  jsr ppu_clear_oam

  jsr draw_step_dispatch
  jsr ppu_wait_vblank
  jsr blit_step_dispatch
  ldx #0
  lda #>OAM
  stx OAMADDR
  sta OAM_DMA
  lda #VBLANK_NMI|BG_0000|OBJ_1000
  ldy #0
  sec
  jsr ppu_screen_on
  jsr update_sound
  jsr read_pads

  lda new_keys
  and #KEY_LEFT
  beq notLeft
  lda #6
  jsr start_sound
  lda #5
  jsr start_sound
  
  lda cur_page
  bne :+
  lda num_pages
  sta cur_page
:
  dec cur_page
  lda #DRAWS_TABS
  sta draw_step
  jsr move_to_end_of_page
notLeft:

  lda new_keys
  and #KEY_RIGHT
  beq notRight
  lda #6
  jsr start_sound
  lda #5
  jsr start_sound
  lda #DRAWS_TABS
  sta draw_step
  ldx cur_page
  inx
  cpx num_pages
  bcc :+
  ldx #0
:
  stx cur_page
  jsr move_to_start_of_page
notRight:

  lda new_keys
  and #KEY_UP
  beq notUp
  lda cur_titleno
  bne notUpWrap
  ldy num_pages
  lda (tab_data_ptr),y
  sta cur_titleno
notUpWrap:
  dec cur_titleno
  jsr move_to_page_with_title
notUp:

  lda new_keys
  and #KEY_DOWN
  beq notDown
  inc cur_titleno
  lda cur_titleno
  ldy num_pages
  cmp (tab_data_ptr),y
  bcc notDownWrap
  lda #0
  sta cur_titleno
notDownWrap:
  jsr move_to_page_with_title
notDown:

  lda new_keys
  and #KEY_A
  beq notA
  lda showing_description
  bne done
  lda #1
  sta showing_description
  lda #DRAWS_TABS
  sta draw_step
notA:

  lda new_keys
  and #KEY_B
  beq notB
  lda showing_description
  beq notB
  lda #0
  sta showing_description
  lda #DRAWS_TABS
  sta draw_step
notB:

  lda new_keys
  and #KEY_START
  bne done
  jmp forever
done:
  jsr stop_music
  jsr update_sound
  lda cur_titleno
  rts  
.endproc

.proc move_to_page_with_title
  lda #5
  jsr start_sound
  lda showing_description
  beq not_showing_desc

  ; If showing the description, always reset the draw step to tabs.
  lda #DRAWS_TABS
  bne have_draw_step
not_showing_desc:

  ; If showing the title list, reset the draw step to arrow
  ; unless the current draw step is TITLELIST.
  lda draw_step
  cmp #DRAWS_TITLELIST
  beq noDrawReset
  lda #DRAWS_ARROW
have_draw_step:
  sta draw_step
noDrawReset:

  ; We need to find the first page where the page start is greater
  ; than the titleno, and then use the page before that.
  ldy #1
srchloop:
  lda (tab_data_ptr),y
  cmp cur_titleno
  beq not_found
  bcs found
not_found:
  iny
  cpy num_pages
  bcc srchloop
found:
  dey
  cpy cur_page
  beq right_page
  sty cur_page
  lda #DRAWS_TABS
  sta draw_step
  lda #6
  jsr start_sound
right_page:
  rts
.endproc

.proc move_to_start_of_page
  ldy cur_page
  tya
  beq :+
  lda (tab_data_ptr),y
:
  sta cur_titleno
  rts
.endproc

.proc move_to_end_of_page
  ldy cur_page
  iny
  lda (tab_data_ptr),y
  sec
  sbc #1
  sta cur_titleno
  rts
.endproc


;;
; Point tab_data_ptr at the first tab's title.
.proc iter_tab_titles
  lda PAGELIST
  sta tab_data_ptr
  lda PAGELIST+1
  sta tab_data_ptr+1
  ldy #0
  lda (tab_data_ptr),y
  sta num_pages
  sec
  adc tab_data_ptr
  sta tab_data_ptr
  bcc :+
  inc tab_data_ptr+1
:
  rts
.endproc

.proc setup_card_bg

  ; First fill the palette
  lda #$3F
  sta PPUADDR
  ldx #$00
  stx PPUADDR
  stx PPUMASK
:
  lda card_palette,x
  sta PPUDATA
  inx
  cpx #32
  bcc :-
  
  ; clear background to black
  ldx #$20
  lda #$00
  tay
  jsr ppu_clear_nt

  ; Load non-text tiles used for select screen
  ldy #$00
  sty PPUADDR
  sty PPUADDR
  lda #<select_tiles_chr
  sta 0
  lda #>select_tiles_chr
  sta 1
  ldx #2
copyseltilesloop:
  lda (0),y
  sta PPUDATA
  iny
  bne copyseltilesloop
  inc 1
  dex
  bne copyseltilesloop

  ; Clear the text area for now
  lda #$06
  sta PPUADDR
  stx PPUADDR
  lda #$FF
clrtxt:
  .repeat 10
    sta PPUDATA
  .endrepeat
  dex
  bne clrtxt

  ; Steps to get the menu up:
  ; 1. Load the attribute table
  ; 2. Find width of each tab in tiles
  ; 3. Draw text to tabs.
  ; 4. Draw tab headers to nametable.
  ; 5. Draw card including TV frame.
  ; 6. Draw text lines.
  
  ; Attribute table: leftmost 7 units grayscale, rest colored for
  ; 10 lines of text on top and 10 on bottom
  ; 01234567890123456789012345678901
  ; 0 0 0 0 0 0 0 x x x x x x x x x
  lda #$23
  sta PPUADDR
  lda #$C8
  sta PPUADDR
  ldx #0
attrloop:
  lda #$00
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  lda card_attrtable,x
  and #%11001100  ; left half zero, right half xx
  sta PPUDATA
  lda card_attrtable,x
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  inx
  cpx #6
  bcc attrloop


  ; Now find the width of each tab's title.
  ; I could pre-cache tab title widths in Python, but I felt like
  ; writing it in 6502 asm instead.  If the menu segment fills up,
  ; I might consider moving it back out to Python instead.
  jsr iter_tab_titles
  ldx #0
calcwid_width = 3
calcwid_strlen = 4
calcwidloop:

  ; First measure in pixels.
  stx draw_progress
  clc
  ldy tab_data_ptr
  lda tab_data_ptr+1
  jsr vwfStrWidth  ; deposits pointer to string in $00-$01
  sta calcwid_width
  sty calcwid_strlen

  ; If the remainder after dividing by 8 is one, see if the
  ; rightmost pixel can be ignored by looking at the glyph.
  and #$07
  cmp #1
  bne notRemainderOne

  ; actually look at the last glyph using vwfGlyphWidth
  ; (which trashes $06-$07)
  dey
  lda ($00),y
  jsr vwfGlyphWidth
cmploop:
  dex
  asl a
  bne cmploop
  cpx #$00
  beq notRemainderOne
  bmi notRemainderOne
  txa
  eor #$FF
  sec
  adc calcwid_width
  sta calcwid_width
notRemainderOne:

  ; Divide by 8 and round up, giving tile count.
  lda calcwid_width
  lsr a
  adc #3
  lsr a
  lsr a
  ldx draw_progress
  sta tab_tilelens,x

  ; Multiply by 8, subtract the width to get X offset.
  asl a
  asl a
  asl a
  sec
  sbc calcwid_width
  lsr a
  sta tab_xoffsets,x
  lda calcwid_strlen
  sec
  adc tab_data_ptr
  sta tab_data_ptr
  bcc :+
  inc tab_data_ptr+1
:
  inx
  cpx num_pages
  bcc calcwidloop

  ; TO DO: Fail if more than 8 tabs, more than 24 tiles, or
  ; if (2 * tabs) + tiles > 28.

  ; Now draw the tabs' text.
  jsr iter_tab_titles
  ldx #0
tab_draw_pass:
tiles_filled_this_pass = 2
drawing_tab = 3
  stx draw_progress  ; at the start of this pass
  stx drawing_tab
  lda #0
  sta tiles_filled_this_pass
  jsr clearLineImg

  ; Now draw as many as will fit in 16 tiles, each at
  ; x = 8 * running tile count so far + X offset
tab_draw_one:

  ; After $C120
  lda tiles_filled_this_pass
  asl a
  asl a
  asl a
  ldx drawing_tab
  adc tab_xoffsets,x
  sta 0
  clc
  lda tiles_filled_this_pass
  adc tab_tilelens,x
  cmp #16
  bcs this_pass_done
  sta tiles_filled_this_pass
  clc
  ldy tab_data_ptr
  lda tab_data_ptr+1
  ldx 0
  jsr vwfPuts
  iny
  bne :+
  clc
  adc #1
:
  sty tab_data_ptr
  sta tab_data_ptr+1
  inc drawing_tab
  lda drawing_tab
  cmp num_pages
  bcc tab_draw_one
this_pass_done:

  ; Invert tiles (white=1, black=0)
  lda tiles_filled_this_pass
  jsr invertTiles

  ; Set the VRAM destination based on the total tile width of
  ; previous tabs and blit.
  ldx draw_progress
  lda #$48  ; first tile number
  dex
  bmi noprevtabs
addprevtabs:
  clc
  adc tab_tilelens,x
  dex
  bpl addprevtabs
noprevtabs:
  sta 0
  lda #0
  sec
  rol 0
  rol a
  asl 0
  rol a
  asl 0
  rol a
  asl 0
  rol a
  ldy 0
  jsr copyLineImg

  ldx drawing_tab
  cpx num_pages
  bcc tab_draw_pass
  
  ; End of shit that could (should?) have been done statically
  ; in Python

  jsr update_sound

  ; Now with the tiles set up, we FINALLY get to start
  ; drawing the nametable.
  ; Rows 0-2: black
  ; Row 3: tab headers
  ; Row 4: underline non-selected tabs
  ; Row 5: fullwidth white
  ; Rows 6-25: Card body
  ; Row 26: Bottom of card
  ; Rows 27-29: black

  ; For each tab draw one $08, a number of unique tiles starting at
  ; $48 corresponding to the tile width, and one $09.
  lda #VBLANK_NMI
  sta PPUCTRL
  ldx #0
  stx draw_progress
  lda #$20
  sta PPUADDR
  lda #$62
  sta PPUADDR
  ldy #$48
each_nttab:
  lda tab_tilelens,x
  sta 0
  lda #$08
  sta PPUDATA
nttabloop:
  sty PPUDATA
  iny
  dec 0
  bne nttabloop
  lda #$09
  sta PPUDATA
  inx
  cpx num_pages
  bcc each_nttab

  ; Blank row above card body
  lda #$20
  sta PPUADDR
  lda #$A1
  sta PPUADDR
  lda #$02
  ldx #30
blankrowattoploop:
  sta PPUDATA
  dex
  bne blankrowattoploop

  ; To draw card body:
  ; 31, 0: black
  ; 1-2: white
  ; 3-12: tv
  ; 13: white
  ; 14: arrow
  ; 15-30: text
  ; 31: black
cardbodyloop:
  stx draw_progress
  lda #$00
  sta PPUDATA
  sta PPUDATA
  lda #$02
  sta PPUDATA
  sta PPUDATA
  cpx #10
  bcc :+
  ldx #9
:
  ldy tv_rows,x
  ldx #10
tvloop:
  lda tv_shape,y
  sta PPUDATA
  iny
  dex
  bne tvloop
  ldx #2
  stx PPUDATA
  inx
  stx PPUDATA
  
  ; now we have the left half of this row of the card done;
  ; the rest is the text half
  lda draw_progress
  cmp #10
  bcc :+
  sbc #10
:
  asl a
  asl a
  asl a
  asl a
  adc #$60
  tax
  ldy #16
texthalfloop:
  stx PPUDATA
  inx
  dey
  bne texthalfloop

  ldx draw_progress
  inx
  cpx #20
  bcc cardbodyloop

  sty PPUDATA
  sty PPUDATA
  lda #$06
  sta PPUDATA
  lda #$02
  ldx #28
blankrowatbottomloop:
  sta PPUDATA
  dex
  bne blankrowatbottomloop
  lda #$07
  sta PPUDATA
  rts
.endproc

; Updating the screen while the menu is on ;;;;;;;;;;;;;;;;;;;;;;;;;;

;;
; Loads a pointer to a title directory record.
; XY not modified.
;
; The titledir format is
; 0: PRG bank number
; 1: CHR bank number
; 2: Screenshot number
; 3: Year minus 1970
; 4: Number of players type
; 5-7: unused
; 8: 2-byte offset to title and author within the name block
; 10: 2-byte offset to description within the description block
; 12: 2-byte reset vector
; 14-15: unused
; @param A title number
; @return pointer in $00-$01
.proc get_titledir_a
  sta 0
  lda #$00
  sta 1
  lda 0
  .repeat 4
  asl a
  rol 1
  .endrepeat
  adc TITLELIST
  sta 0
  lda 1
  adc TITLELIST+1
  sta 1
  rts
.endproc

;;
; Prepares a buffer to be copied to the screen, based on the current
; draw_step and draw_progress.
.proc draw_step_dispatch
  ldx draw_step
  lda steps+1,x
  pha
  lda steps,x
  pha
doNothing:
  rts
steps:
  .addr doNothing-1
  .addr doNothing-1
  .addr draw_step_titlelist-1
  .addr draw_step_description-1
  .addr doNothing-1
  .addr draw_step_screenshot-1
.endproc

;;
; Copies a buffer to the screen, based on the current
; draw_step and draw_progress.
.proc blit_step_dispatch
  ldx draw_step
  lda steps+1,x
  pha
  lda steps,x
  pha
doNothing:
  rts
steps:
  .addr doNothing-1
  .addr blit_step_tabborder-1
  .addr blit_step_vwf-1
  .addr blit_step_vwf-1
  .addr blit_step_arrow-1
  .addr blit_step_screenshot-1
.endproc

;;
; Draws the bottom border of each tab, which shows the user
; which pane is selected.
.proc blit_step_tabborder
  ; For each tab draw two more than the tile width, in blank ($02)
  ; if the tab is selected or card top border ($0A) if not.
  lda #VBLANK_NMI
  sta PPUCTRL
  ldx #0
  stx draw_progress
  lda #$20
  sta PPUADDR
  lda #$81
  sta PPUADDR
  lda #$04
  sta PPUDATA
  ldy #28
  sty 0
each_nttab:
  ldy tab_tilelens,x
  iny
  iny
  lda #$02  ; white
  cpx cur_page
  beq nttabloop
  lda #$0A  ; white with overline
nttabloop:
  sta PPUDATA
  dec 0
  dey
  bne nttabloop
  inx
  cpx num_pages
  bcc each_nttab

  ; fill in extra space to the right of the rightmost tab
  ldx 0
  beq trcorner
  bmi no_extra_overline
  lda #$0A
finishloop:
  sta PPUDATA
  dex
  bne finishloop
trcorner:
  lda #$05
  sta PPUDATA
no_extra_overline:

  lda #0
  sta draw_progress
  
  lda #DRAWS_TITLELIST
  ldy showing_description
  beq have_next_step
  lda #DRAWS_DESCRIPTION
  bne have_next_step

arrow_only:
  lda #0
  sta draw_progress
  lda cur_titleno
  eor screenshot_titleno
  beq have_next_step
  lda #DRAWS_SCREENSHOT
have_next_step:
  sta draw_step
  lda cur_titleno
  ldy cur_page
  cpy #0
  beq :+
  sbc (tab_data_ptr),y
:
  tay  ; Y = number of rows until selected game
  iny
  
  ; hide the arrow if the description is showing
  lda showing_description
  beq not_hide_for_description
  ldy #0
not_hide_for_description:
  ldx #20
  lda #VBLANK_NMI|VRAM_DOWN
  sta PPUCTRL
  lda #$20
  sta PPUADDR
  lda #$CE
  sta PPUADDR
loop:
  lda #$03
  dey
  bne notArrowHere
  lda #$01
notArrowHere:
  sta PPUDATA
  dex
  bne loop
  rts
.endproc
blit_step_arrow = blit_step_tabborder::arrow_only

.proc draw_step_titlelist
  jsr clearLineImg
  clc
  lda cur_page
  tay
  beq :+
  lda (tab_data_ptr),y
:
  adc draw_progress
  iny
  cmp (tab_data_ptr),y
  bcs past_last_title_on_page
  
  jsr get_titledir_a
have_titledir:
  lda #20
  sta num_lines_on_page
  clc
  ldy #8
  lda NAMESLIST
  adc (0),y
  sta 2
  iny
  lda NAMESLIST+1
  adc (0),y
  ldy 2
  ldx #0
  jsr vwfPuts
  sta desc_data_ptr+1
  sty desc_data_ptr
  jmp line_ready

past_last_title_on_page:
  lda num_lines_on_page
  cmp #20
  bcc line_ready
  lda draw_progress
  sta num_lines_on_page

line_ready:
  ; now copy the line
  lda #16
  jmp invertTiles
.endproc

.proc blit_step_vwf
  .if 0
    lda nmis
    sta $0178
    lda num_lines_on_page
    sta $017A
    lda prev_lines_on_page
    sta $017C
  .endif
  lda draw_progress
  cmp num_lines_on_page
  bcc not_last_row
  cmp prev_lines_on_page
  bcc not_last_row
  ldy num_lines_on_page
  sty prev_lines_on_page
  ldy #DRAWS_ARROW
  sty draw_step  
not_last_row:
  ldy #$00
  cmp #10
  bcc not2ndhalf
  sbc #10
  ldy #$08
not2ndhalf:
  clc
  adc #$06
  inc draw_progress
  jmp copyLineImg
.endproc

.proc draw_step_description
  jsr clearLineImg
  lda cur_titleno
  jsr get_titledir_a

  lda draw_progress
  beq draw_step_titlelist::have_titledir
  cmp #1
  beq continue_title
  cmp #2
  beq do_players
  cmp #4
  beq start_description
  bcs continue_description
  jmp line_done
continue_title:
  ldy #'2'
  sty interbank_fetch_buf
  ldy #'0'
  sty interbank_fetch_buf+1
  ldy #0
  sty interbank_fetch_buf+4
  ldy #3
  lda (0),y
  sec
  sbc #2000-1970
  bcs not_before_2000
  dec interbank_fetch_buf
  ldy #'9'
  sty interbank_fetch_buf+1
  adc #256-100
not_before_2000:
  jsr bcd8bit
  ora #'0'
  sta interbank_fetch_buf+3
  lda 0
  and #$0F
  ora #'0'
  sta interbank_fetch_buf+2
  lda 0
  lsr a
  lsr a
  lsr a
  lsr a
  clc
  adc interbank_fetch_buf+1
  sta interbank_fetch_buf+1
  lda #>interbank_fetch_buf
  ldy #<interbank_fetch_buf
  ldx #0
  jsr vwfPuts

  lda desc_data_ptr+1
  ldy desc_data_ptr
  ldx #24
  bne have_line_ptr_with_x
do_players:
  ldy #4
  lda (0),y
  asl a
  tax
  lda numplayers_names+1,x
  ldy numplayers_names,x
have_line_ptr:
  ldx #0
have_line_ptr_with_x:
  jsr vwfPuts
  jmp line_done

start_description:
  ldy #10
  clc
  lda (0),y
  adc DESCSLIST
  sta desc_data_ptr
  iny
  lda (0),y
  adc DESCSLIST+1
  sta desc_data_ptr+1

continue_description:
  lda desc_data_ptr
  sta 0
  lda desc_data_ptr+1
  sta 1
  lda #<DESCSBANK
  sta 2
  lda #>DESCSBANK
  sta 3
  lda #48
  sta 4
  lda #10
  sta interbank_fetch_buf+48
  jsr interbank_fetch
  lda interbank_fetch_buf+0
  bne not_eot
  lda num_lines_on_page
  cmp #20
  bcc line_done
  lda draw_progress
  sta num_lines_on_page
  jmp line_done
not_eot:
  lda #>interbank_fetch_buf
  ldy #<interbank_fetch_buf
  ldx #0
  jsr vwfPuts
  tya
  sec
  sbc #<interbank_fetch_buf
  clc
  adc desc_data_ptr
  sta desc_data_ptr
  bcc line_done
  inc desc_data_ptr+1
line_done:
  lda #16
  jmp invertTiles
.endproc

.proc draw_step_screenshot
  ; screenshot id is titledir[0]
  ; screenshot directory entry format:
  ; bank, addr lo, addr hi, pal 3f11, pal 3f12, pal 3f13

  lda cur_titleno
  jsr get_titledir_a
  ldy #2
  lda (0),y
  sta 0
  ldy #0
  sty 1
  asl a
  adc 0
  rol 1
  asl a
  rol 1
  clc
  adc SCREENSLIST
  sta 2
  lda 1
  adc SCREENSLIST+1
  sta 3
  lda #144  ; including 12.5% overhead for run-free tiles
  sta 4

  lda draw_progress
  bne already_started
  lda cur_titleno
  ora #$80
  sta screenshot_titleno
  ldy #1
  lda (2),y
  sta desc_data_ptr
  iny
  lda (2),y
  sta desc_data_ptr+1
  .repeat 3, I
    iny
    lda (2),y
    sta screenshot_colors+I
  .endrepeat
already_started:
  lda desc_data_ptr
  sta 0
  lda desc_data_ptr+1
  sta 1
  jsr interbank_fetch
  
  lda #<interbank_fetch_buf
  sta ciSrc
  lda #>interbank_fetch_buf
  sta ciSrc+1
  lda #128
  sta ciBufEnd
  asl a
  sta ciBufStart
  jsr unpb53_some
  lda draw_progress
  cmp #6
  sec
  lda ciSrc
  sbc #<interbank_fetch_buf
  clc
  adc desc_data_ptr
  sta desc_data_ptr
  bcc :+
  inc desc_data_ptr+1
:
  rts
.endproc

.proc blit_step_screenshot
  ldx #0
  stx 0
  lda #VBLANK_NMI
  sta PPUCTRL
  lda draw_progress
  lsr a
  ror 0
  ora #$10
  sta PPUADDR
  lda 0
  sta PPUADDR
  ; carry is clear coming out of ror 0
loop:
  ; and coming out of the loop
  .repeat 8, I
    lda PB53_outbuf+I,x
    sta PPUDATA
  .endrepeat
  txa
  adc #8
  tax
  cpx #128
  bcc loop
  
  inc draw_progress
  lda draw_progress
  cmp #7
  bcc not_finished
  lda screenshot_titleno
  and #$7F
  sta screenshot_titleno
  lda #DRAWS_IDLE
  sta draw_step
  lda #$3F
  sta PPUADDR
  lda #$11
  sta PPUADDR
  .repeat 3, I
    lda screenshot_colors+I
    sta PPUDATA
  .endrepeat
not_finished:
  rts
.endproc


SCREENSHOT_LEFT = 32
SCREENSHOT_TOP = 56
.proc setup_oam_for_screenshot
ty = 0
tn = 1
tx = 3

  lda #SCREENSHOT_TOP - 1
  sta ty
  lda #0
  sta tn
  ldx oam_used
rowloop:
  lda #SCREENSHOT_LEFT
  sta tx
tileloop:
  lda ty
  sta OAM,x
  inx
  lda tn
  inc tn
  sta OAM,x
  inx
  lda #0
  sta OAM,x
  inx
  lda tx
  sta OAM,x
  inx
  clc
  adc #8
  sta tx
  cmp #64+SCREENSHOT_LEFT
  bcc tileloop
  lda #7  ; carry set; adding 8
  adc ty
  sta ty
  cmp #55+SCREENSHOT_TOP
  bcc rowloop
  stx oam_used
  rts
.endproc

.segment "RODATA"
select_tiles_chr: .incbin "obj/nes/select_tiles.chr"
card_palette:
  ;    grayscale        unused           bit 0            bit 1
  .byt $0F,$00,$10,$30, $0F,$16,$16,$16, $0F,$10,$0F,$10, $0F,$0F,$10,$10
  ;    screenshot       unused           unused           unused
  .byt $0F,$1a,$2a,$30, $0F,$16,$16,$16, $0F,$16,$16,$16, $0F,$16,$16,$16

tv_shape:
  .byt $0B,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0C,$0D
  .byt $0E,$00,$00,$00,$00,$00,$00,$00,$00,$0F
  .byt $10,$11,$12,$12,$13,$14,$15,$12,$16,$17
  .byt $02,$02,$02,$02,$02,$02,$02,$02,$02,$02
tv_rows: .byt 0, 10, 10, 10, 10, 10, 10, 10, 20, 30
  
card_attrtable:
  .byt $A0,$AA,$AA,$FF,$FF,$0F

.if 0
tab_title_data:
  .byt 5
  .byt 1, 3, 5, 7, 9
  .byt "1-18",0
  .byt "19-36",0
  .byt "37-53",0
  .byt "Toys",0
  .byt "Analysis",0

theme_text:
  .incbin "src/theme.txt"
  .byt 0
.endif

numplayers_names:
  .addr numplayers_1, numplayers_2only, numplayers_12, numplayers_12alt
  .addr numplayers_13, numplayers_14
numplayers_1:      .byt "1 player",0
numplayers_2only:  .byt "2 players ONLY",0
numplayers_12:     .byt "1 or 2 players",0
numplayers_12alt:  .byt "1 or 2 players alternating",0
numplayers_13:     .byt "1 to 3 players",0
numplayers_14:     .byt "1 to 4 players",0

